#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhumenu.h"

char *plugin_name = "IceWM";

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i < n; i++) fprintf(f, "\t");
}

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		if (*s == '\\' || *s == '"') fputc('\\', f);
		fputc(*s++, f);
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s != '%') { fputc(*s++, f); continue; }
		s++;
		if (*s == '%' || *s == '\0') fputc('%', f);
		if (*s != '\0') s++;
	}
}

static void dump (FILE *f, const struct menu *menu, const char *lang, int n)
{
	int i;
	int major = 0;
	char *iconstr;
	for (i = 0; i < menu->nr; i++) {
		if (i > 0 && menu->items[i].major != major) {
			indent(f, n);
			fprintf(f, "separator\n");
		}
		if (!menu->items[i].issubmenu) {
			indent(f, n);
			iconstr = get_value_8bit(&menu->items[i], NULL, "Icon", lang);
			fprintf(f, "prog \"");
			print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\" %s %s",
			  iconstr != NULL ? iconstr : "-",
			  menu->items[i].terminal ? "xterm -e " : "");
			print_exec(f, get_value(&menu->items[i], NULL, "Exec", lang));
			fprintf(f, "\n");
		} else {
			indent(f, n);
			fprintf(f, "menu \"");
			print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\" folder {\n");
			dump(f, menu->items[i].submenu, lang, n+1);
			indent(f, n);
			fprintf(f, "}\n");
		}
		major = menu->items[i].major;
	}
}

int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	if (lang[0] == '\0') lang = "C";
	sprintf(file, "%s/%s", path, lang);
	if ((f = fopen(file, "w")) == NULL) {
		fprintf(stderr, "icewm: fopen %s: %s\n", file, strerror(errno));
		return -1;
	}
	if (menu->nr > 0) dump(f, menu->items[0].submenu, lang, 0);
	fclose(f);
	return 0;
}

